// Generated by ResgenSwift.Analytics 1.2

import MatomoTracker
import FirebaseAnalytics

// MARK: - Protocol

protocol AnalyticsManagerProtocol {
    func logScreen(
        name: String,
        path: String,
        params: [String: Any]?
    )

    func logEvent(
        name: String,
        action: String,
        category: String,
        params: [String: Any]?
    )

    func setEnable(_ enable: Bool)
}

// MARK: - Matomo

class MatomoAnalyticsManager: AnalyticsManagerProtocol {
    
    // MARK: - Properties

    private var tracker: MatomoTracker

    // MARK: - Init

    init(siteId: String, url: String) {
        debugPrint("[Matomo service] Server URL: \(url)")
        debugPrint("[Matomo service] Site ID: \(siteId)")
        tracker = MatomoTracker(
            siteId: siteId,
            baseURL: URL(string: url)!
        )
        
        #if DEBUG
            tracker.dispatchInterval = 5
        #endif
        
        #if DEBUG
            tracker.logger = DefaultLogger(minLevel: .verbose)
        #endif

        debugPrint("[Matomo service] Configured with content base: \(tracker.contentBase?.absoluteString ?? "-")")
        debugPrint("[Matomo service] Opt out: \(tracker.isOptedOut)")
    }
    
    // MARK: - Methods

    func logScreen(
        name: String,
        path: String,
        params: [String: Any]?
    ) {
        guard let trackerUrl = tracker.contentBase?.absoluteString else { return }

        let urlString = URL(string: "\(trackerUrl)" + "/" + "\(path)" + "iOS")
        tracker.track(
            view: [name],
            url: urlString
        )
    }

    func logEvent(
        name: String,
        action: String,
        category: String,
        params: [String: Any]?
    ) {
        tracker.track(
            eventWithCategory: category,
            action: action,
            name: name,
            number: nil,
            url: nil
        )
    }

    func setEnable(_ enable: Bool) {
        tracker.isOptedOut = !enable
    }
}

// MARK: - Firebase

class FirebaseAnalyticsManager: AnalyticsManagerProtocol {
    func logScreen(
        name: String,
        path: String,
        params: [String: Any]?
    ) {
        var parameters = [
            AnalyticsParameterScreenName: name as NSObject
        ]

        if path.isEmpty == false {
            parameters["path"] = path + "/iOS" as NSObject
        }

        if let supplementaryParameters = params {
            for (newKey, newValue) in supplementaryParameters {
                if parameters.contains(where: { (key: String, value: NSObject) in
                    key == newKey
                }) {
                    continue
                }
                
                parameters[newKey] = newValue as? NSObject
            }
        }

        Analytics.logEvent(
            AnalyticsEventScreenView,
            parameters: parameters
        )
    }

    func logEvent(
        name: String,
        action: String,
        category: String,
        params: [String: Any]?
    ) {
        var parameters: [String:NSObject] = [
            AnalyticsParameterItemName: name.replacingOccurrences(of: " ", with: "_") as NSObject
        ]

        if category.isEmpty == false {
            parameters["AnalyticsParameterItemCategory"] = category as NSObject
        }
        
        if action.isEmpty == false {
            parameters["action"] = action as NSObject
        }
        
        if let supplementaryParameters = params {
            for (newKey, newValue) in supplementaryParameters {
                if parameters.contains(where: { (key: String, value: NSObject) in
                    key == newKey
                }) {
                    continue
                }
                
                parameters[newKey] = newValue as? NSObject
            }
        }
        
        Analytics.logEvent(
            AnalyticsEventSelectContent,
            parameters: parameters
        )
    }
    func setEnable(_ enable: Bool) {
        Analytics.setAnalyticsCollectionEnabled(enable)
    }
}

// MARK: - Manager

class AnalyticsManager {
    static var shared = AnalyticsManager()
    
    // MARK: - Properties

    var managers: [TargetType: AnalyticsManagerProtocol] = []

    private var isEnabled: Bool {
        if ProcessInfo.processInfo.environment["XCODE_RUNNING_FOR_PREVIEWS"] == "1" {
            false
        } else {
            true
        }
    }

    // MARK: - Methods

    private func setAnalytics(enable: Bool) {
        managers.forEach { (key, value) in
            if analytics.contains(where: { type in
                type == key
            }) {
                value.setEnable(enable)
            }
        }
    }

    func enableAnalytics(_ analytics: [TargetType] = TargetType.allCases) {
        setAnalytics(enable: true)
    }

    func disableAnalytics(_ analytics: [TargetType] = TargetType.allCases) {
        setAnalytics(enable: false)
    }

    func configure(siteId: String, url: String) {
        managers[TrackerType.matomo] = MatomoAnalyticsManager(
            siteId: siteId,
            url: url
        )
        managers[TrackerType.firebase] = FirebaseAnalyticsManager()
    }
        
    private func logScreen(
        name: String,
        path: String,
        params: [String: Any]?
    ) {
        guard isEnabled else { return }

        managers.values.forEach { manager in
            manager.logScreen(
                name: name,
                path: path,
                params: params
            )
        }
    }
    
    private func logEvent(
        name: String,
        action: String,
        category: String,
        params: [String: Any]?
    ) {
        guard isEnabled else { return }

        managers.values.forEach { manager in
            manager.logEvent(
                name: name,
                action: action,
                category: category,
                params: params
            )
        }
    }

    // MARK: - section_one

    static func logScreenS1DefOne(title: String) {
        AnalyticsManager.shared.logScreen(
            name: "s1 def one \(title)",
            path: "s1_def_one/\(title)",
            params: nil
        )
    }

    static func logEventS1DefTwo(
        title: String,
		count: String,
		test2: String = "test"
    ) {
        AnalyticsManager.shared.logEvent(
            name: "s1 def two",
            action: "test",
            category: "test",
            params: [
                "title": title,
				"count": count,
				"test": "test",
				"test2": test2
            ]
        )
    }

    // MARK: - section_two

    static func logScreenS2DefOne() {
        AnalyticsManager.shared.logScreen(
            name: "s2 def one",
            path: "s2_def_one/",
            params: nil
        )
    }
}
